from django import forms

from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.libs.fields import Function
from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw
from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Pre Process' )
report.save()

section = Section(report=report, title='Section 0')
section.save()

TableField.create ('choices', 'Choices', section,
                   field_cls = forms.ChoiceField,
                   pre_process_func =
                   Function(funcs.preprocess_field_choices))

TableField.create ('choices_with_params', 'Choices with params', section,
                   field_cls = forms.ChoiceField,
                   pre_process_func =
                   Function(funcs.preprocess_field_choices_with_params,
                            params={'start' : 1,
                                    'end': 3,
                                    'prefix': 'pre'}))

a = AnalysisTable.create('test-criteria-preprocess', tables={},
                         function=funcs.analysis_echo_criteria)
a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from rest_framework import serializers
from steelscript.appfwk.apps.datasource.models import Table, Column, Job


#
# Field serializers
#
class PickledObjectField(serializers.Field):
    def field_to_native(self, obj, fieldname):
        field = getattr(obj, fieldname)
        #field = dbsafe_decode(field)
        if field and 'func' in field:
            field['func'] = repr(field['func'])
        return field


class JobDataField(serializers.Field):
    def field_to_native(self, obj, fieldname):
        try:
            return obj.values()
        except AttributeError:
            # requesting data before its ready
            # XXX what is the best choice to do here?
            return {}


#
# Model serializers
#
class TableSerializer(serializers.ModelSerializer):
    options = PickledObjectField()

    class Meta:
        model = Table


class ColumnSerializer(serializers.ModelSerializer):
    options = PickledObjectField()

    class Meta:
        model = Column
        fields = ('id', 'name', 'label', 'position', 'options', 'iskey',
                  'synthetic', 'datatype', 'units')


class JobListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Job
        fields = ('id', 'table', 'criteria', 'actual_criteria', 'status', 'message', 'progress', 'remaining')
        read_only_fields = ('id', 'status', 'message', 'progress', 'remaining')


class JobSerializer(serializers.ModelSerializer):
    criteria = PickledObjectField()
    actual_criteria = PickledObjectField()

    class Meta:
        model = Job
        fields = ('id', 'table', 'criteria', 'actual_criteria', 'status', 'message', 'progress', 'remaining')
        read_only_fields = ('id', 'status', 'message', 'progress', 'remaining')


class JobDataSerializer(serializers.ModelSerializer):
    data = JobDataField()

    class Meta:
        model = Job
        fields = ('data',)



from steelscript.appfwk.apps.report.models import Report, Section
import steelscript.appfwk.apps.report.modules.raw as raw
from steelscript.appfwk.apps.datasource.forms import fields_add_time_selection
from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Time Selection' )
report.save()

section = Section(report=report, title='Section 0')
section.save()

a = AnalysisTable.create('test-criteria-timeselection', tables={},
                         function=funcs.analysis_echo_criteria)
fields_add_time_selection(a, initial_duration='1 day')

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from re import compile

from django.http import HttpResponseRedirect
from django.conf import settings
from rest_framework.views import exception_handler
from rest_framework.exceptions import NotAuthenticated

from steelscript.appfwk.project.utils import get_request


#
# Global authentication locks
# adapted from stack overflow question
# http://tinyurl.com/kaeqg37
#
def get_exempts():
    exempts = [compile(settings.LOGIN_URL.lstrip('/'))]
    if hasattr(settings, 'LOGIN_EXEMPT_URLS'):
        exempts += [compile(expr) for expr in settings.LOGIN_EXEMPT_URLS]
    return exempts


class LoginRequiredMiddleware(object):
    """
    Middleware that requires a user to be authenticated to view any page other
    than reverse(LOGIN_URL_NAME). Exemptions to this requirement can optionally
    be specified in settings via a list of regular expressions in
    LOGIN_EXEMPT_URLS (which you can copy from your urls.py).

    Requires authentication middleware and template context processors to be
    loaded. You'll get an error if they aren't.
    """
    def process_request(self, request):
        assert hasattr(request, 'user'), "The Login Required middleware\
requires authentication middleware to be installed. Edit your\
MIDDLEWARE_CLASSES setting to insert\
'django.contrib.auth.middlware.AuthenticationMiddleware'. If that\
doesn't work, ensure your TEMPLATE_CONTEXT_PROCESSORS setting includes\
'django.core.context_processors.auth'."
        if not request.user.is_authenticated():
            path = request.path.lstrip('/')
            if not any(m.match(path) for m in get_exempts()):
                return HttpResponseRedirect(
                    settings.LOGIN_URL + "?next=" + request.path)


#
# Custom exception handling for Django REST Framework
#
def authentication_exception_handler(exc):
    """ Returns redirect to login page only when requesting HTML. """
    request = get_request()

    if (isinstance(exc, NotAuthenticated) and
            'text/html' in request.negotiator.get_accept_list(request)):
        return HttpResponseRedirect(settings.LOGIN_URL + "?next=" + request.path)

    response = exception_handler(exc)

    return response



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from . import criteria_functions as funcs

report = Report(title='Criteria Circular Dependency')
report.save()

# Section
section = Section(report=report, title='Section 0')
section.save()

a = AnalysisTable.create('test-criteria-circulardependency', tables={},
                         function=funcs.analysis_echo_criteria)

TableField.create(keyword='t1', obj=a,
                  post_process_template='table_computed:{t2}',
                  hidden=False)

TableField.create(keyword='t2', obj=a,
                  post_process_template='table_computed:{t3}',
                  hidden=False)

TableField.create(keyword='t3', obj=a,
                  post_process_template='table_computed:{t1}',
                  hidden=False)

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



from django import forms

from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.libs.fields import Function
from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw
from . import criteria_functions as funcs

report = Report(title='Criteria Changing with Sections',
                field_order =['first', 'second'])
report.save()

section = Section.create(report=report, title='Section 0', section_keywords=['first','second'])
section.save()

a = AnalysisTable.create('test-criteria-changingchoiceswithsections-0', tables={},
                         function=funcs.analysis_echo_criteria)
TableField.create ('first', 'First Choice', a,
                   field_cls = forms.ChoiceField,
                   field_kwargs = {'choices': (('a', 'Option A'),
                                               ('b', 'Option B') ) })

TableField.create ('second', 'Second Choice', a,
                   field_cls = forms.ChoiceField,
                   pre_process_func = Function(funcs.preprocess_changesecond),
                   dynamic=True)

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table 0')

section = Section.create(report=report, title='Section 1', section_keywords=['first','second'])
section.save()

a = AnalysisTable.create('test-criteria-changingchoiceswithsections-1', tables={},
                         function=funcs.analysis_echo_criteria)
TableField.create ('first', 'First Choice', a,
                   field_cls = forms.ChoiceField,
                   field_kwargs = {'choices': (('a', 'Option A'),
                                               ('b', 'Option B') ) })

TableField.create ('second', 'Second Choice', a,
                   field_cls = forms.ChoiceField,
                   pre_process_func =
                   Function(funcs.preprocess_changesecond),
                   dynamic=True)

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table 1')



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.libs.fields import Function

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Shared Fields' )
report.save()

x = TableField.create('x', 'X Value')
for i in range(2):

    section = Section(report=report, title='Section %d' % i)
    section.save()

    a = AnalysisTable.create('test-criteria-sharedfields-%d' % i, tables={},
                             function = funcs.analysis_echo_criteria)
    a.add_column('key', 'Key', iskey=True, datatype="string")
    a.add_column('value', 'Value', datatype="string")

    a.fields.add(x)
    y = TableField.create('y', 'Y Value', a,
                          hidden=True,
                          parent_keywords = ['x'],
                          post_process_func = Function(funcs.sharedfields_compute,
                                                       params={'factor': 10*(i+1)}))

    raw.TableWidget.create(section, a, 'Table %d' % i)



from steelscript.appfwk.apps.datasource.modules.analysis import \
     AnalysisTable, AnalysisQuery

class WhoisTable(AnalysisTable):
    class Meta:
        proxy = True

    _query_class = 'WhoisQuery'

    def post_process_table(self, field_options):
        super(WhoisTable, self).post_process_table(field_options)
        self.copy_columns(self.options.tables['t'])
        self.add_column('whois', label="Whois link", datatype='html')


def make_whois_link(ip):
    return ('<a href="http://whois.arin.net/rest/nets;q=%s?showDetails=true&'
            'showARIN=false&ext=netref2" target="_blank">Whois record</a>' % ip)

class WhoisQuery(AnalysisQuery):

    def post_run(self):
        """ Return a data frame that simply adds a whois link for each IP. """
        df = self.tables['t']
        df['whois'] = df['host_ip'].map(make_whois_link)
        self.data = df
        return True



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from django.contrib import admin

from steelscript.appfwk.apps.report.models import Report, Widget, WidgetJob


class ReportAdmin(admin.ModelAdmin):
    list_display = ('enabled', 'title', 'position', 'namespace',
                    'sourcefile', 'slug')
    fieldsets = (
        (None, {
            'fields': ('title', 'position', 'enabled', 'slug',
                       'namespace', 'sourcefile', 'field_order',
                       'hidden_fields')
        }),
        ('Auto Reports', {
            'fields': ('hide_criteria', 'reload_minutes')
        }),
        ('Report Fields', {
            'classes': ('collapse',),
            'fields': ('fields',)
        })
    )
    filter_horizontal = ('fields',)
    actions = ['mark_enabled', 'mark_disabled']

    def mark_disabled(self, request, queryset):
        queryset.update(enabled=False)
    mark_disabled.short_description = 'Mark selected reports disabled'

    def mark_enabled(self, request, queryset):
        queryset.update(enabled=True)
    mark_enabled.short_description = 'Mark selected reports enabled'

admin.site.register(Report, ReportAdmin)


class WidgetAdmin(admin.ModelAdmin):
    list_display = ('title', 'section', 'module', 'uiwidget')
    list_filter = ('section', 'module', 'uiwidget', )

#admin.site.register(Widget, WidgetAdmin)


class WidgetJobAdmin(admin.ModelAdmin):
    pass

#admin.site.register(WidgetJob, WidgetJobAdmin)



import glob
import time
import os
import json
import logging

from django.test import TestCase, Client
from django.core import management
from django.utils.datastructures import SortedDict

from steelscript.appfwk.apps.datasource.models import Job
from steelscript.appfwk.apps.report.models import Report, Widget

logger = logging.getLogger(__name__)


class ReportRunnerTestCase(TestCase):

    report = None

    @classmethod
    def setUpClass(cls):
        localdir = os.path.dirname(__file__)
        initial_data = glob.glob(os.path.join(localdir, '*.json'))
        management.call_command('loaddata', *initial_data)

        if not isinstance(cls.report, list):
            reports = [cls.report]
        else:
            reports = cls.report

        for report in reports:
            cls.load_report(report)

    @classmethod
    def load_report(cls, report):
            path = 'steelscript.appfwk.apps.report.tests.reports.' + report
            logger.info("Loading report: %s" % path)
            management.call_command('reload', report_name=path)

    def setUp(self):
        logger.info('Logging in as admin')
        logger.info('Report count: %d' % len(Report.objects.all()))
        self.client = Client()
        self.assertTrue(self.client.login(username='admin', password='admin'))

    def check_status(self, response, expect_fail):
        sc = int(response.status_code)
        if (sc >= 400 and sc < 500):
            logger.info("Report error response code %s\n%s" %
                        (sc, response.content))
            self.assertTrue(expect_fail)
            return

        self.assertTrue(not expect_fail)
        self.assertEqual(response.status_code, 200)

    def run_report(self, criteria, report=None,
                   expect_fail_report=False, expect_fail_job=False):
        if report is None:
            report = self.report

        logger.info("Running report %s with criteria:\n%s" %
                    (report, criteria))

        try:
            response = self.client.post('/report/rvbd_portal/%s/' % report,
                                        data=criteria)
        except:
            self.assertTrue(expect_fail_report)
            return

        self.check_status(response, expect_fail_report)
        if expect_fail_report:
            return

        report_data = json.loads(response.content)
        logger.debug("Report data:\n%s" % json.dumps(report_data, indent=2))

        widgets = SortedDict()
        for widget_data in report_data[1:]:
            wid = widget_data['widgetid']
            widget = Widget.objects.get(id=wid)
            logger.info('Processing widget %s' % widget)

            # POST to the widget url to create the WidgetJob instance
            widget_url = widget_data['posturl']
            widget_criteria = widget_data['criteria']
            postdata = {'criteria': json.dumps(widget_criteria)}
            logger.debug("Widget post data:\n%s" % (json.dumps(postdata, indent=2)))
            response = self.client.post(widget_url, data=postdata)
            self.check_status(response, expect_fail_job)

            if expect_fail_job:
                return

            widgetjob_data = json.loads(response.content)
            logger.debug("Widget response data:\n%s" % (json.dumps(widgetjob_data, indent=2)))

            # Extract the job url and get the first response
            joburl = widgetjob_data['joburl']
            widgets[joburl] = None

        for joburl in widgets:
            while True:
                response = self.client.get(joburl)
                job_data = json.loads(response.content)
                widgets[joburl] = job_data
                logger.debug("Widget job data:\n%s" % (json.dumps(job_data, indent=2)))
                if widgets[joburl]['status'] not in [Job.COMPLETE, Job.ERROR]:
                    time.sleep(0.1)
                else:
                    break

        return widgets



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import logging

import pandas

from steelscript.appfwk.apps.datasource.models import\
     DatasourceTable, Column, Job, Table, TableQueryBase, BatchJobRunner
from steelscript.appfwk.libs.fields import Function


logger = logging.getLogger(__name__)


class AnalysisException(Exception):
    def _init__(self, message, *args, **kwargs):
        self.message = message
        super(AnalysisException, self).__init__(*args, **kwargs)


class AnalysisTable(DatasourceTable):
    """
    An AnalysisTable builds on other tables, running them first to collect
    data, then extracting the data as pandas.DataFrame objects.  The
    set of DataFrames is then passed to a user defined function that
    must return a DataFrame with columns matching the Columns associated
    with this Table.

    `tables` is hashmap of dependent tables, mapping a names expected
        by the analysis functon to table ids

    `function` is a pointer to the user defined analysis function, or
        a Function object which includes parameters

    For example, consider an input of two tables A and B, and an
    AnalysisTable that simply concatenates A and B:

        A = Table.create('A')
        Column.create(A, 'host')
        Column.create(A, 'bytes')

        B = Table.create('B')
        Column.create(B, 'host')
        Column.create(B, 'pkts')

        from config.reports.helpers.analysis_funcs import combine_by_host

        Combined = AnalysisTable('Combined',
                                 tables = {'t1': A,
                                           't2': B},
                                 function = combine_by_host)
        Combined.add_column('host')
        Combined.add_column('bytes')
        Combined.add_column('pkts')

    Then in config/reports/helpers/analysis_func.py

        def combine_by_host(dst, srcs):
            # Get the pandas.DataFrame objects for t1 and t2
            t1 = srcs['t1']
            t2 = srcs['t2']

            # Now create a new DataFrame that joins these
            # two tables by the 'host'
            df = pandas.merge(t1, t2, left_on='host', right_on='host',
                              how='outer')
            return df

    Note that the function must defined in a separate file in the 'helpers'
    directory.
    """
    class Meta: proxy = True

    _ANALYSIS_TABLE_OPTIONS = {
        'tables': None,            # dependent tables to be run first
        'related_tables': None}    # related tables that are reference only

    _ANALYSIS_FIELD_OPTIONS = {
        'copy_fields': True }      # If true, copy TableFields from tables
                                   # and related_tables

    _query_class = 'AnalysisQuery'

    @classmethod
    def process_options(cls, table_options):
        # handle direct id's, table references, or table classes
        # from tables option and transform to simple table id value
        for i in ['tables', 'related_tables']:
            for k, v in (table_options[i] or {}).iteritems():
                table_options[i][k] = Table.to_ref(v)

        return table_options

    def post_process_table(self, field_options):
        if field_options['copy_fields']:
            keywords = set()
            for i in ['tables', 'related_tables']:
                refs = self.options[i] or {}
                for ref in refs.values():
                    table = Table.from_ref(ref)
                    for f in table.fields.all():
                        if f.keyword not in keywords:
                            self.fields.add(f)
                            keywords.add(f.keyword)


class AnalysisQuery(TableQueryBase):

    def run(self):
        deptables = self.table.options.tables
        if not deptables:
            self.tables = {}
            return True

        # Collect all dependent tables
        options = self.table.options

        # Create dataframes for all dependent tables
        tables = {}

        logger.debug("%s: dependent tables: %s" % (self, deptables))
        depjobids = {}
        max_progress = (len(deptables)*100.0/float(len(deptables)+1))
        batch = BatchJobRunner(self.job, max_progress=max_progress)

        for (name, ref) in deptables.items():
            deptable = Table.from_ref(ref)
            job = Job.create(
                table=deptable,
                criteria=self.job.criteria.build_for_table(deptable)
            )
            batch.add_job(job)
            logger.debug("%s: starting dependent job %s" % (self, job))
            depjobids[name] = job.id

        batch.run()

        logger.debug("%s: all dependent jobs complete, collecting data"
                     % str(self))

        failed = False
        for (name, id) in depjobids.items():
            job = Job.objects.get(id=id)

            if job.status == job.ERROR:
                self.job.mark_error("Dependent Job failed: %s" % job.message)
                failed = True
                break

            f = job.data()
            tables[name] = f
            logger.debug("%s: Table[%s] - %d rows" %
                         (self, name, len(f) if f is not None else 0))

        if failed:
            return False

        self.tables = tables

        logger.debug("%s: deptables completed successfully" % (self))
        return True


class CriteriaTable(AnalysisTable):
    class Meta: proxy = True

    _query_class = 'CriteriaQuery'

    def post_process_table(self, field_options):
        super(CriteriaTable, self).post_process_table(field_options)

        self.add_column('key', 'Criteria Key', iskey=True,
                        datatype=Column.DATATYPE_STRING)
        self.add_column('value', 'Criteria Value',
                        datatype=Column.DATATYPE_STRING)

class CriteriaQuery(AnalysisQuery):

    def post_run(self):
        criteria = self.job.criteria
        values = [[str(k),str(v)]
                  for k,v in criteria.iteritems()]
        values.append(['criteria.starttime', str(criteria.starttime)])
        df = pandas.DataFrame(values,
                              columns=['key', 'value']).sort('key')

        self.data = df
        return True



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from cStringIO import StringIO
import os
import glob
import optparse
import imp

from django.core.management.base import BaseCommand
from django.core import management
from django.conf import settings
from django import db
from django.db import transaction, DatabaseError

from steelscript.appfwk.apps.preferences.models import SystemSettings

# list of files/directories to ignore
IGNORE_FILES = ['helpers']


class Command(BaseCommand):
    args = None
    help = ('Reset the database. Prompts for confirmation unless '
            '`--force` is included as an argument.')

    option_list = BaseCommand.option_list + (
        optparse.make_option('--force',
                             action='store_true',
                             dest='force',
                             default=False,
                             help='Ignore reset confirmation.'),
        optparse.make_option('--drop-users',
                             action='store_true',
                             dest='drop_users',
                             default=False,
                             help='Drop all locally created users, only '
                                  'default admin account will be enabled '
                                  'afterwards. Default will keep all user '
                                  'accounts across reset.'),
    )

    def save_users(self):
        """ Store user definitions to buffer in memory rather than disk. """
        self.stdout.write('Saving existing users ... ', ending='')
        try:
            buf = StringIO()
            management.call_command('dumpscript', 'preferences', stdout=buf)
            buf.seek(0)
            clean_buf = buf.read().replace('<UTC>', 'pytz.UTC')
            clean_buf = clean_buf.replace('import datetime\n',
                                          'import datetime\nimport pytz\n')
            self.user_buffer = clean_buf
        except DatabaseError:
            self.user_buffer = None

        db.close_connection()
        self.stdout.write('done.')

    def load_users(self):
        """ Load stored user module and run it, creating new user objects.

        This script is run under a transaction to avoid committing partial
        settings in case of some exception.
        """
        # ref http://stackoverflow.com/a/14192708/2157429
        if self.user_buffer is not None:
            self.stdout.write('Loading saved users ... ', ending='')
            m = imp.new_module('runscript')
            exec self.user_buffer in m.__dict__
            with transaction.commit_on_success():
                m.run()
            db.close_connection()
            self.stdout.write('done.')

    def handle(self, *args, **options):
        if not options['force']:
            msg = ('You have requested to reset portal, this will delete\n'
                   'everything from the database and start from scratch.\n'
                   'Are you sure?\n'
                   "Type 'yes' to continue, or 'no' to cancel: ")
            confirm = raw_input(msg)
        else:
            confirm = 'yes'

        if confirm != 'yes':
            self.stdout.write('Aborting.')
            return

        self.user_buffer = None
        if not options['drop_users']:
            self.save_users()

        # lets clear it
        self.stdout.write('Resetting database ... ', ending='')
        management.call_command('reset_db',
                                interactive=False,
                                router='default')
        self.stdout.write('done.')

        management.call_command('clean',
                                applications=False,
                                report_id=None,
                                clear_cache=True,
                                clear_logs=True)

        management.call_command('clean_pyc', path=settings.PROJECT_ROOT)

        management.call_command('syncdb', interactive=False)

        self.stdout.write('Loading initial data ... ', ending='')
        initial_data = glob.glob(os.path.join(settings.INITIAL_DATA, '*.json'))
        initial_data.sort()
        if not options['drop_users']:
            # filter out default admin user fixure and reload previous users
            initial_data = [f for f in initial_data if 'admin_user' not in f]

        if initial_data:
            management.call_command('loaddata', *initial_data)
        self.load_users()

        # if we don't have a settings fixture, create new default item
        if not SystemSettings.objects.all():
            SystemSettings().save()

        management.call_command('reload', report_id=None)



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import datetime
import logging
import tempfile

import dateutil.parser
from django.test import Client

from steelscript.appfwk.apps.datasource.models import Job
from steelscript.appfwk.apps.report.models import Report, Widget, Section
from . import reportrunner
from steelscript.appfwk.libs.call_command import call_command


logger = logging.getLogger(__name__)

class CriteriaTest(reportrunner.ReportRunnerTestCase):

    def run_with_criteria(self, criteria, expected=None, report=None,
                          expect_fail_report=False, expect_fail_job=False):
        self.run_report_with_criteria(criteria, report=report, expected=expected,
                                      expect_fail_report=expect_fail_report,
                                      expect_fail_job=expect_fail_job)
        if not expect_fail_report and not expect_fail_job:
            self.run_tables_with_criteria(criteria, expected=expected)

    def run_report_with_criteria(self, criteria, expected=None, report=None,
                                 expect_fail_report=False, expect_fail_job=False):
        """ Run the report via views, much like the UI. """
        if report is None:
            report = self.report

        # If not provided, expected result should match input criteria
        if expected is None:
            expected = [criteria]
        elif not isinstance(expected, list):
            expected = [expected]

        # Perform a little magic on endtime because the way report
        # processing works, the endtime is actuall split into two pieces
        # by the django forms processor
        run_criteria = {}
        for k,v in criteria.iteritems():
            if k == 'endtime':
                (e0, e1) = v.split(' ',1)
                run_criteria['endtime_0'] = e0
                run_criteria['endtime_1'] = e1
            else:
                run_criteria[k] = v

        # Now, run the report and get back all the widget data indexed
        # by job url
        widgets = self.run_report(run_criteria, report,
                                  expect_fail_report=expect_fail_report,
                                  expect_fail_job=expect_fail_job)

        if expect_fail_report or expect_fail_job:
            return

        # Verify the results
        for i,e in enumerate(expected):
            w = widgets.values()[i]
            self.assertEqual(w['status'], Job.COMPLETE,
                             'Widget %d, message %s' % (i, w['message']))

            returned_criteria = dict(w['data'])
            logger.debug("Widget %d, returned_criteria: %s" % (i, returned_criteria))

            for k,v in e.iteritems():
                self.assertEqual(returned_criteria[k], v,
                                 "Key %s => %s vs %s" %
                                 (k, v, returned_criteria[k]))

    def run_tables_with_criteria(self, criteria, expected=None, report=None,
                                 expect_fail_report=False, expect_fail_job=False):
        """ Run the tables via the command line run_table command. """
        if report is None:
            report = self.report

        if expected is None:
            expected = [criteria]
        elif not isinstance(expected, list):
            expected = [expected]

        report = Report.objects.get(slug = report)
        tables = []
        for w in Widget.objects.filter(section__in = report.section_set.all()):
            tables.extend(w.tables.all())

        for i,t in enumerate(tables):

            with tempfile.NamedTemporaryFile() as outfile:
                filename = outfile.name

            r = call_command('table',
                             table_id=t.id,
                             as_csv=True,
                             output_file=filename,
                             criteria=['%s:%s' % (k,v) for (k,v) in criteria.iteritems()])

            data = []
            with open(filename, 'r') as f:
                for line in f:
                    data.append(line.strip().split(',',1))

            returned_criteria = {}
            for k,v in data[1:]:
                returned_criteria[k] = v

            logger.debug("Table %s, returned_criteria: %s" % (t.id, returned_criteria))

            for k,v in expected[i].iteritems():
                self.assertEqual(returned_criteria[k], v,
                                 "Key %s => %s vs %s" %
                                 (k, v, returned_criteria[k]))

class CriteriaTimeSelection(CriteriaTest):

    report = 'criteria_timeselection'

    def test_duration_1day(self):
        self.run_with_criteria({'endtime': '12/1/2013 11:00 am +0000',
                                'duration': '1 week'},
                               {'duration': str(datetime.timedelta(days=7)),
                                'starttime': str(dateutil.parser.parse("11/24/2013 11:00am +0000")),
                                'endtime': str(dateutil.parser.parse("12/1/2013 11:00am +0000"))})

    def test_duration_5min(self):
        self.run_with_criteria({'endtime': '12/1/2013 11:00 am +0000',
                                'duration': '5 min'},
                               {'duration': str(datetime.timedelta(seconds=60*5)),
                                'starttime': str(dateutil.parser.parse("12/1/2013 10:55am +0000")),
                                'endtime': str(dateutil.parser.parse("12/1/2013 11:00am +0000"))})

    def test_bad_time(self):
        self.run_with_criteria({'endtime': '12/1f/2013 11:00 am +0000',
                                'duration': '5 min'},
                               expect_fail_report=True)


class CriteriaPreProcess(CriteriaTest):

    report = 'criteria_preprocess'

    def test(self):
        self.run_with_criteria({'choices' : 'val1',
                                'choices_with_params' : 'pre_val1'})

        self.run_with_criteria({'choices' : 'val2',
                                'choices_with_params' : 'pre_val3'})



class CriteriaPostProcess(CriteriaTest):

    report = 'criteria_postprocess'

    def test(self):
        self.run_with_criteria({'w' : '1', 'x' : '2', 'y': '5'},
                               {'wx' : '3', 'wy' : '6', 'xy': '7'})

class CriteriaSharedFields(CriteriaTest):

    report = 'criteria_sharedfields'

    def test(self):
        self.run_with_criteria({'x': '2'},
                               [ {'x': '2', 'y': '14'},
                                 {'x': '2', 'y': '24'} ])

class CriteriaPostProcessErrors(CriteriaTest):

    report = 'criteria_postprocesserrors'

    def test_no_error(self):
        self.run_with_criteria({'error': 'none'},
                               {'x' : '1'})

    def test_syntax(self):
        self.run_with_criteria({'error': 'syntax'},
                               expect_fail_job=True)


class CriteriaParents(CriteriaTest):

    report = 'criteria_parents'

    def test(self):
        self.run_with_criteria({'report_independent': 'top'},

                               {'report_independent': 'top',
                                'report_computed' : 'report_computed:top',
                                'section_computed' : 'section_computed:report_computed:top',
                                'table_computed' : 'table_computed:section_computed:report_computed:top'})

class CriteriaDefaults(CriteriaTest):

    report = 'criteria_defaults'

    def test(self):
        self.run_with_criteria({}, {}, expect_fail_report=True)

        self.run_with_criteria({'report-2': 'r22'},

                               {'report-1': 'r1',
                                'report-2': 'r22',
                                'section-1': 's1',
                                'section-2': 's2',
                                'table-1': 't1',
                                'table-2': 't2'})

        self.run_with_criteria({'report-1': 'r11',
                                'report-2': 'r22'},

                               {'report-1': 'r11',
                                'report-2': 'r22',
                                'section-1': 's1',
                                'section-2': 's2',
                                'table-1': 't1',
                                'table-2': 't2'})

        self.run_with_criteria({'report-1': 'r11',
                                'report-2': 'r22',
                                'table-1': 't11'},

                               {'report-1': 'r11',
                                'report-2': 'r22',
                                'section-1': 's1',
                                'section-2': 's2',
                                'table-1': 't11',
                                'table-2': 't2'})


class CriteriaSectionKeywords(CriteriaTest):

    report = 'criteria_sectionkeywords'

    def test(self):
        report = Report.objects.get(slug__endswith=self.report)
        section0 = Section.objects.get(report=report, title="Section 0")
        section1 = Section.objects.get(report=report, title="Section 1")
        self.run_report_with_criteria({'__s%s_k1' % section0.id: 'v1',
                                       '__s%s_k1' % section1.id: 'v2'},

                                      [{'k1' : 'v1'},
                                       {'k1' : 'v2'}])


class CriteriaChangingChoices(CriteriaTest):

    report = 'criteria_changingchoices'

    def test(self):
        self.run_with_criteria({'first' : 'a',
                                'second' : 'a-1'})

        self.run_with_criteria({'first' : 'b',
                                'second' : 'b-2'})

        self.run_with_criteria({'first' : 'b',
                                'second' : 'a-1'},
                               expect_fail_report=True)

        self.run_with_criteria({'first' : 'a',
                                'second' : 'b-2'},
                               expect_fail_report=True)

class CriteriaChangingChoicesWithSections(CriteriaTest):

    report = 'criteria_changingchoiceswithsections'

    def test(self):
        report = Report.objects.get(slug__endswith=self.report)
        section0 = Section.objects.get(report=report, title="Section 0")
        section1 = Section.objects.get(report=report, title="Section 1")
        self.run_report_with_criteria({'__s%s_first' % section0.id: 'a',
                                       '__s%s_second' % section0.id : 'a-1',
                                       '__s%s_first' % section1.id: 'b',
                                       '__s%s_second' % section1.id : 'b-2'},

                                      [{'first' : 'a', 'second' : 'a-1'},
                                       {'first' : 'b', 'second' : 'b-2'} ])

class CriteriaCircularDependency(CriteriaTest):

    report = 'criteria_circulardependency'

    def test(self):
        self.run_report_with_criteria({}, expect_fail_report=True)

class CriteriaTwoReports(CriteriaTest):

    report = ['criteria_tworeports_1', 'criteria_tworeports_2']

    def test(self):
        self.run_report_with_criteria({'k1': '2'},
                                      {'k1': '2', 'k2': 'r2'},
                                      report=self.report[1])

        self.load_report(self.report[0])

        # Need to re-login
        self.client = Client()
        self.client.login(username='admin', password='admin')

        self.run_report_with_criteria({'k1': '2'},
                                      {'k1': '2', 'k2': 'r2'},
                                      report=self.report[1])



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import json
import logging

from django.http import Http404, HttpResponse
from django.contrib import messages
from rest_framework.permissions import IsAdminUser
from rest_framework.renderers import TemplateHTMLRenderer
from rest_framework.views import APIView
from rest_framework.response import Response

from steelscript.appfwk.apps.plugins import plugins

logger = logging.getLogger(__name__)


class PluginsListView(APIView):
    """ Display list of installed plugins """

    renderer_classes = (TemplateHTMLRenderer, )
    permission_classes = (IsAdminUser,)

    def get(self, request):
        changed = request.QUERY_PARAMS.get('changed', False)

        return Response({'plugins': list(plugins.all()),
                         'changed': changed},
                        template_name='plugins_list.html')


class PluginsDetailView(APIView):
    """ Display detail of specific plugin """

    renderer_classes = (TemplateHTMLRenderer, )
    permission_classes = (IsAdminUser,)

    def get(self, request, slug, *args, **kwargs):
        try:
            plugin = plugins.get(slug)
        except KeyError:
            return Http404

        return Response({'plugin': plugin})

    def post(self, request, slug, *args, **kwargs):
        """ Enable or disable plugin - rest of details are read-only """
        try:
            plugin = plugins.get(slug)
        except KeyError:
            return Http404

        enabled = request.DATA.get('enabled', False)

        # since we don't have helpful form cleaning, check for json 'false' too
        if (enabled == 'false' or enabled is False) and plugin.can_disable:
            plugin.enabled = False
            msg = 'Plugin %s disabled.' % plugin.title
        else:
            plugin.enabled = True
            msg = 'Plugin %s enabled.' % plugin.title

        messages.add_message(request, messages.INFO, msg)

        return HttpResponse(json.dumps({'plugin': plugin.__dict__}))



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.libs.fields import Function

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Post Process Errors' )
report.save()

section = Section(report=report, title='Section 0')
section.save()

a = AnalysisTable.create('test-criteria-postprocess', tables={},
                         function=funcs.analysis_echo_criteria)

TableField.create('error', 'Error type', a)
TableField.create('x', 'X Value', a, hidden=True,
                  post_process_func = Function(funcs.postprocesserrors_compute))

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Defaults')
report.save()

# Report-level criteria
TableField.create(keyword='report-1', label='Report 1', obj=report, initial='r1')
TableField.create(keyword='report-2', label='Report 2', obj=report, required=True)

# Section
section = Section(report=report, title='Section 0')
section.save()

# Section-level criteria
TableField.create(keyword='section-1', label='Section 1', obj=section, initial='s1')
TableField.create(keyword='section-2', label='Section 2', obj=section, required=True, initial='s2')

# Table
a = AnalysisTable.create('test-criteria-postprocess', tables={},
                         function=funcs.analysis_echo_criteria)

# Table-level criteria
TableField.create(keyword='table-1', label='Table 1', obj=a, initial='t1')
TableField.create(keyword='table-2', label='Table 2', obj=a, initial='t2')


a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Parents', hidden_fields = ['report_computed',
                                                           'section_computed',
                                                           'table_computed'] )
report.save()

# Report-level independent
TableField.create(keyword='report_independent', label='Report Independent', obj=report)

# Report-level computed
TableField.create(keyword='report_computed', obj=report,
                  post_process_template='report_computed:{report_independent}',
                  hidden=False)

# Section
section = Section(report=report, title='Section 0')
section.save()

# Section-level computed
TableField.create(keyword='section_computed', obj=section,
                  post_process_template='section_computed:{report_computed}',
                  hidden=False)

# Table
a = AnalysisTable.create('test-criteria-postprocess', tables={},
                         function=funcs.analysis_echo_criteria)

# Table-level computed
TableField.create(keyword='table_computed', obj=a,
                  post_process_template='table_computed:{section_computed}',
                  hidden=False)

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.libs.fields import Function

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from steelscript.appfwk.apps.report.tests.reports import criteria_functions as funcs

report = Report(title='Criteria Post Process' )
report.save()

section = Section(report=report, title='Section 0')
section.save()

a = AnalysisTable.create('test-criteria-postprocess', tables={},
                         function=funcs.analysis_echo_criteria)

TableField.create('w', 'W Value', a)
TableField.create('x', 'X Value', a)
TableField.create('y', 'Y Value', a)

for (f1,f2) in [('w', 'x'), ('w', 'y'), ('x', 'y')]:
    ( TableField.create
      ('%s%s' % (f1, f2), '%s+%s Value' % (f1, f2), a,
       hidden = True, parent_keywords=[f1, f2],
       post_process_func = Function(funcs.postprocess_field_compute,
                                    params={'fields': [f1, f2]})))

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

from steelscript.netprofiler.appfwk.datasources.netprofiler import (NetProfilerTimeSeriesTable,
                                                                   NetProfilerGroupbyTable)

report = Report.create("QoS Report", position=11)

interface_field = TableField.create(keyword='interface', label='Interface',
                                    required=True)
datafilter_field = TableField.create(keyword='datafilter', hidden=True,
                                     post_process_template='interfaces_a,{interface}')

report.add_section("Overall")

# Define a Overall TimeSeries showing In/Out Utilization
p = NetProfilerTimeSeriesTable.create('qos-overall-util',
                                      duration=15, resolution=60,
                                      interface=True)
p.fields.add(interface_field)
p.fields.add(datafilter_field)

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('in_avg_util', 'Avg Inbound Util %', units='B/s')
p.add_column('out_avg_util', 'Avg Outbound Util %', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Overall Utilization", width=12)

# Define a Overall TimeSeries showing In/Out Totals
p = NetProfilerTimeSeriesTable.create('qos-overall-total',
                                      duration=15, resolution=15 * 60,
                                      interface=True)
p.fields.add(interface_field)
p.fields.add(datafilter_field)

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('in_total_bytes', 'Total Inbound Bytes', units='B/s')
p.add_column('out_total_bytes', 'Total Outbound Bytes', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Overall In/Out Bandwidth", width=6)

# Define a Overall TimeSeries showing In/Out Totals
p = NetProfilerTimeSeriesTable.create('qos-overall-avg',
                                      duration=15, resolution=60,
                                      interface=True)
p.fields.add(interface_field)
p.fields.add(datafilter_field)

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('in_avg_bytes', 'Avg Inbound Bytes/s', units='B/s')
p.add_column('out_avg_bytes', 'Avg Outbound Bytes/s', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p,
                  "Overall Average In/Out Bandwidth", width=6)

###
# QOS Summary Tables
for direction in ['inbound', 'outbound']:
    p = NetProfilerGroupbyTable.create('qos-%s-totals' % direction,
                                       groupby='qos',
                                       duration=15, resolution=60,
                                       interface=True)
    p.fields.add(interface_field)

    TableField.create(
        keyword='%s_filterexpr' % direction,
        obj=p,
        hidden=True,
        post_process_template='%s interface {interface}' % direction
    )
    p.fields_add_filterexprs_field('%s_filterexpr' % direction)

    p.add_column('qos', 'QoS', iskey=True)
    p.add_column('qos_name', 'QoS Name', iskey=True)
    p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')
    p.add_column('total_bytes', 'Total Bytes/s', units='B/s')
    p.add_column('avg_util', 'Avg Util', units='pct')
    p.add_column('peak_util', 'Peak Util', units='pct')

    report.add_widget(yui3.TableWidget, p,
                      "%s Traffic by QoS" % direction.capitalize(), width=6)

###
# QoS sections, defaults to AF11, EF, and Default
for i, qos in enumerate(['AF11', 'EF', 'Default']):

    report.add_section("QoS %d" % i)

    ###
    # QOS Tables

    for direction in ['inbound', 'outbound']:
        p = NetProfilerTimeSeriesTable.create('qos-%d-%s' % (i, direction),
                                              duration=15, resolution=60,
                                              interface=True)
        p.fields.add(interface_field)
        p.fields.add(datafilter_field)
        qos_field = TableField.create(keyword='qos_%d' % i,
                                      label='QoS %d (DSCP)' % i, obj=p,
                                      initial=qos)
        TableField.create(
            keyword='%s_filterexpr' % direction, obj=p,
            hidden=True,
            post_process_template=('%s interface {interface} and qos {qos_%d}' %
                                   (direction, i))
        )
        p.fields_add_filterexprs_field('%s_filterexpr' % direction)

        p.add_column('time', 'Time', datatype='time', iskey=True)
        p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')

        report.add_widget(yui3.TimeSeriesWidget, p,
                          "QoS {qos_%d} - Average %s Bandwidth" % (i, direction.capitalize()),
                          width=6)



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import os
import sys
import glob
import shutil
import inspect
import pkg_resources

from steelscript.common.exceptions import RvbdHTTPException, RvbdException


def get_request():
    """ Run up the stack and find the `request` object. """
    # XXX see discussion here:
    #    http://nedbatchelder.com/blog/201008/global_django_requests.html
    # alternative would be applying middleware for thread locals
    # if more cases need this behavior, middleware may be better option

    frame = None
    try:
        for f in inspect.stack()[1:]:
            frame = f[0]
            code = frame.f_code
            if code.co_varnames[:1] == ("request",):
                return frame.f_locals["request"]
            elif code.co_varnames[:2] == ("self", "request",):
                return frame.f_locals["request"]
    finally:
        del frame


def get_module():
    """ Run up the stack, find the `module` object and return its name.

    This works reasonably well in non-interactive environments when a file
    is being explicitly imported, but will return '__main__' when called
    directly or from an interactive session, like IPython.
    """
    frame, frm, mod = None, None, None

    try:
        for frame in inspect.stack()[1:]:
            if frame[3] == '<module>':
                frm = frame[0]
                mod = inspect.getmodule(frm)
                # interactive shells will have <module> frame with
                # no corresponding module, skip them
                if hasattr(mod, '__name__'):
                    return mod.__name__
        return None
    finally:
        del frame, frm, mod


def get_sourcefile(modname):
    """Return sourcefile name for given module name."""
    if modname is not None:
        return modname
    else:
        return 'default'


def get_namespace(sourcefile):
    """Return namespace for given sourcefile. """
    ns = sourcefile.split('.')

    if (sourcefile.startswith('reports.') and (ns[1] == 'default' or
                                               len(ns) == 2)):
        # 'reports.default', 'reports.overall', etc.
        namespace = 'default'
    elif sourcefile.startswith('custom_reports'):
        namespace = 'custom'
    elif ns[0] == 'steelscript':
        # 'steelscript.wireshark.appfwk.plugin' --> 'wireshark'
        namespace = ns[1]
    elif len(ns) >= 3:
        # take the middle portion, should align to plugin names
        # handles subdirectories as well
        namespace = '.'.join(ns[1:-1])
    else:
        # some other folder structure
        namespace = 'custom'

    return namespace


def get_caller_name(frames_back=2):
    """ Determine filename of calling function.
        Used to determine source of Report class definition.
    """
    frame = inspect.stack()[frames_back]
    frm = frame[0]
    mod = inspect.getmodule(frm)
    del frm
    return mod.__name__


def link_pkg_dir(pkgname, src_path, dest_dir, symlink=True,
                 replace=True, buf=None):
    """Create a link from a resource within an installed package.

    :param pkgname: name of installed package
    :param src_path: relative path within package to resource
    :param dest_dir: absolute path to location to link/copy
    :param symlink: create a symlink or copy files
    :param replace: if True, will unlink/delete before linking
    :param buf: IO buffer to send debug statements, if None uses sys.stdout
    """
    if buf is None:
        debug = sys.stdout.write
    else:
        debug = buf

    src_dir = pkg_resources.resource_filename(pkgname, src_path)

    if os.path.islink(dest_dir) and not os.path.exists(dest_dir):
        debug(' unlinking %s ...\n' % dest_dir)
        os.unlink(dest_dir)

    if os.path.exists(dest_dir):
        if not replace:
            return

        if os.path.islink(dest_dir):
            debug(' unlinking %s ...\n' % dest_dir)
            os.unlink(dest_dir)
        else:
            debug(' removing %s ...\n' % dest_dir)
            shutil.rmtree(dest_dir)

    if symlink:
        debug(' linking %s --> %s\n' % (src_dir, dest_dir))
        os.symlink(src_dir, dest_dir)
    else:
        debug(' copying %s --> %s\n' % (src_dir, dest_dir))
        shutil.copytree(src_dir, dest_dir)


def link_pkg_files(pkgname, src_pattern, dest_dir, symlink=True,
                   replace=True, buf=None):
    """Create links for files from a resource within an installed package.

    :param pkgname: name of installed package
    :param src_pattern: relative path within package to file resource,
        can be a glob pattern
    :param dest_dir: absolute path to location to link/copy, symlinks
        will be made inside this dir rather than linking to the dir itself
    :param symlink: create a symlink or copy files
    :param replace: if True, will unlink/delete before linking
    :param buf: IO buffer to send debug statements, if None uses sys.stdout
    """
    if buf is None:
        debug = sys.stdout.write
    else:
        debug = buf

    src_dir = pkg_resources.resource_filename(pkgname, src_pattern)
    src_files = glob.glob(src_dir)

    if os.path.islink(dest_dir):
        # avoid putting files within a symlinked path
        debug(' skipping linked directory %s ...\n' % dest_dir)
        return

    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)

    for f in src_files:
        fn = os.path.basename(f)
        dest_file = os.path.join(dest_dir, fn)

        if os.path.exists(dest_file) and not replace:
            debug(' skipping file %s ...\n' % dest_file)
            continue

        if os.path.islink(dest_file):
            debug(' unlinking %s ...\n' % dest_file)
            os.unlink(dest_file)
        elif os.path.exists(dest_file):
            debug(' removing %s ...\n' % dest_file)
            os.unlink(dest_file)

        if symlink:
            debug(' linking %s --> %s\n' % (f, dest_file))
            os.symlink(f, dest_file)
        else:
            debug(' copying %s --> %s\n' % (f, dest_file))
            shutil.copy(f, dest_file)


# list of files/directories to ignore
IGNORE_FILES = ['helpers']


class Importer(object):
    """ Helper functions for importing modules. """
    def __init__(self, buf=None):
        if buf is None:
            self.stdout = sys.stdout
        else:
            self.stdout = buf

    def import_file(self, f, name):
        try:
            if name in sys.modules:
                reload(sys.modules[name])
                self.stdout.write('reloading %s as %s\n' % (f, name))
            else:
                __import__(name)
                self.stdout.write('importing %s as %s\n' % (f, name))

        except RvbdHTTPException as e:
            instance = RvbdException('From config file "%s": %s\n' %
                                     (name, e.message))
            raise RvbdException, instance, sys.exc_info()[2]

        except SyntaxError as e:
            msg_format = '%s: (file: %s, line: %s, offset: %s)\n%s\n'
            message = msg_format % (e.msg, e.filename,
                                    e.lineno, e.offset, e.text)
            instance = type(e)('From config file "%s": %s\n' % (name,
                                                                message))
            raise type(e), instance, sys.exc_info()[2]

        except Exception as e:
            instance = type(e)('From config file "%s": %s\n' % (name,
                                                                str(e)))
            raise type(e), instance, sys.exc_info()[2]

    def import_directory(self, root, report_name=None, ignore_list=None):
        """ Recursively imports all python files in a directory
        """
        if ignore_list is None:
            ignore_list = IGNORE_FILES

        rootpath = os.path.basename(root)
        for path, dirs, files in os.walk(root):
            for i, d in enumerate(dirs):
                if d in ignore_list:
                    dirs.pop(i)

            for f in files:
                if f in ignore_list or not f.endswith('.py') or '__init__' in f:
                    continue

                f = os.path.splitext(f)[0]
                dirpath = os.path.relpath(path, root)
                if dirpath != '.':
                    name = os.path.join(rootpath, dirpath, f)
                else:
                    name = os.path.join(rootpath, f)
                name = '.'.join(name.split(os.path.sep))

                if report_name and report_name != name:
                    self.stdout.write('skipping %s (%s) ...\n' % (f, name))
                    continue

                self.import_file(f, name)



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import json
import logging

from rest_framework.reverse import reverse
from rest_framework import generics

from steelscript.appfwk.apps.datasource.serializers import (TableSerializer,
                                                     ColumnSerializer,
                                                     JobSerializer,
                                                     JobDataSerializer,
                                                     JobListSerializer)
from steelscript.appfwk.apps.datasource.models import Table, Column, Job, Criteria


logger = logging.getLogger(__name__)


class TableList(generics.ListCreateAPIView):
    model = Table
    serializer_class = TableSerializer
    paginate_by = 20


class TableDetail(generics.RetrieveUpdateDestroyAPIView):
    model = Table
    serializer_class = TableSerializer


class TableColumnList(generics.ListCreateAPIView):
    model = Column
    serializer_class = ColumnSerializer
    paginate_by = 20

    def get_queryset(self):
        """Filter results to specific table."""
        return Column.objects.filter(table=self.kwargs['pk'])


class TableJobList(generics.ListCreateAPIView):
    model = Job
    serializer_class = JobSerializer

    def get_queryset(self):
        """Filter results to specific table."""
        return Job.objects.filter(table=self.kwargs['pk'])

    def post(self, request, *args, **kwargs):
        # auto-populate table info for criteria values
        data = request.DATA.copy()
        data['table'] = self.kwargs['pk']
        request._data = data
        return super(TableJobList, self).post(request, *args, **kwargs)

    def pre_save(self, obj):
        """Populate criteria object with defaults."""
        try:
            criteria = Criteria(table=obj.table,
                                **json.loads(self.request.POST['criteria']))
        except KeyError:
            criteria = Criteria(table=obj.table)

        obj.criteria = criteria

    def post_save(self, obj, created=False):
        # kickoff job once its been created
        if created:
            obj.start()

    def get_success_headers(self, data):
        # override method to return location of new job resource
        try:
            job = self.object
            url = reverse('job-detail', args=(job.pk,), request=self.request)
            return {'Location': url}
        except (TypeError, KeyError):
            return {}


class ColumnList(generics.ListCreateAPIView):
    model = Column
    serializer_class = ColumnSerializer
    paginate_by = 20


class ColumnDetail(generics.RetrieveUpdateDestroyAPIView):
    model = Column
    serializer_class = ColumnSerializer


class JobList(generics.ListCreateAPIView):
    model = Job
    serializer_class = JobListSerializer
    paginate_by = 10

    def post_save(self, obj, created=False):
        if created:
            obj.start()

    def get_success_headers(self, data):
        # override method to return location of new job resource
        try:
            job = self.object
            url = reverse('job-detail', args=(job.pk,), request=self.request)
            return {'Location': url}
        except (TypeError, KeyError):
            return {}


class JobDetail(generics.RetrieveAPIView):
    model = Job
    serializer_class = JobSerializer


class JobDetailData(generics.RetrieveAPIView):
    model = Job
    serializer_class = JobDataSerializer



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw
from steelscript.appfwk.apps.datasource.forms import fields_add_time_selection, fields_add_resolution
from steelscript.appfwk.libs.fields import Function

# Report
from steelscript.appfwk.apps.report.tests.reports import synthetic_functions as funcs

report = Report(title='Synthetic No Resampling')
report.save()

# Section
section = Section(report=report, title='Section 0')
section.save()

# Table
a = AnalysisTable.create('test-synthetic-resampling', tables={},
                         function=Function(funcs.analysis_generate_data,
                                           params={'source_resolution': 60}),
                         resample=True)
fields_add_time_selection(a)
fields_add_resolution(a)

a.add_column('time', 'Time', iskey=True, datatype='time')
a.add_column('value', 'Value')

raw.TableWidget.create(section, a, 'Table')



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import os
import sys
import cgi
import json
import datetime
import importlib
import traceback
import logging

import pytz
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.template import RequestContext
from django.template.defaultfilters import date
from django.shortcuts import render_to_response
from django.core.urlresolvers import reverse
from django.core.servers.basehttp import FileWrapper
from django.core import management
from django.utils.datastructures import SortedDict
from rest_framework import generics, views
from rest_framework.response import Response
from rest_framework.parsers import JSONParser
from rest_framework.renderers import TemplateHTMLRenderer, JSONRenderer

from steelscript.common.timeutils import round_time
from steelscript.appfwk.apps.datasource.models import Job, Table
from steelscript.appfwk.apps.datasource.serializers import TableSerializer
from steelscript.appfwk.apps.datasource.forms import TableFieldForm
from steelscript.appfwk.apps.devices.models import Device
from steelscript.appfwk.apps.geolocation.models import Location, LocationIP
from steelscript.appfwk.apps.preferences.models import SystemSettings
from steelscript.appfwk.apps.report.models import Report, Section, Widget, WidgetJob
from steelscript.appfwk.apps.report.serializers import ReportSerializer
from steelscript.appfwk.apps.report.utils import create_debug_zipfile


logger = logging.getLogger(__name__)


def reload_config(request, namespace=None, report_slug=None):
    """ Reload all reports or one specific report
    """
    if namespace and report_slug:
        logger.debug("Reloading %s report" % report_slug)
        management.call_command('reload',
                                namespace=namespace,
                                report_name=report_slug)
    elif namespace:
        logger.debug("Reloading reports under namespace %s" % namespace)
        management.call_command('reload', namespace=namespace)
    else:
        logger.debug("Reloading all reports")
        management.call_command('reload')

    if ('HTTP_REFERER' in request.META and
        'reload' not in request.META['HTTP_REFERER']):
        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    elif hasattr(request, 'QUERY_PARAMS') and 'next' in request.QUERY_PARAMS:
        return HttpResponseRedirect(request.QUERY_PARAMS['next'])
    else:
        return HttpResponseRedirect(reverse('report-view-root'))


def download_debug(request):
    """ Create zipfile and send it back to client
    """
    # XXX when we implement RBAC this method needs to be ADMIN-level only

    zipfile = create_debug_zipfile()
    wrapper = FileWrapper(file(zipfile))
    response = HttpResponse(wrapper, content_type='application/zip')
    zipname = os.path.basename(zipfile)
    response['Content-Disposition'] = 'attachment; filename=%s' % zipname
    response['Content-Length'] = os.stat(zipfile).st_size
    return response


class ReportView(views.APIView):
    """ Main handler for /report/{id}
    """
    model = Report
    serializer_class = ReportSerializer
    renderer_classes = (TemplateHTMLRenderer, JSONRenderer)

    def get(self, request, namespace=None, report_slug=None):
        # handle REST calls

        queryset = Report.objects.filter(enabled=True)

        if request.accepted_renderer.format != 'html':
            if namespace and report_slug:
                queryset = queryset.get(namespace=namespace,
                                        slug=report_slug)
            elif report_slug:
                queryset = queryset.get(namespace='default',
                                        slug=report_slug)
            elif namespace:
                queryset = queryset.filter(namespace='default')

            serializer = ReportSerializer(instance=queryset)
            return Response(serializer.data)

        # handle HTML calls
        try:
            if namespace is None:
                namespace = queryset[0].namespace

            if report_slug is None:
                qs = queryset.filter(namespace=namespace).order_by('position', 'title')
                kwargs = {'report_slug': qs[0].slug,
                          'namespace': namespace}
                return HttpResponseRedirect(reverse('report-view',
                                                    kwargs=kwargs))
            else:
                report = queryset.get(namespace=namespace, slug=report_slug)
        except:
            raise Http404

        logging.debug('Received request for report page: %s' % report_slug)

        devices = Device.objects.all()
        if len(devices) == 0:
            # redirect if no devices defined
            return HttpResponseRedirect(reverse('device-list'))

        # search across all enabled devices
        for device in devices:
            if (device.enabled and ('host.or.ip' in device.host or
                                    device.username == '<username>' or
                                    device.password == '<password>' or
                                    device.password == '')):
                return HttpResponseRedirect('%s?invalid=true' %
                                            reverse('device-list'))

        if not request.user.profile_seen:
            # only redirect if first login
            return HttpResponseRedirect(reverse('preferences')+'?next=/report')

        # factory this to make it extensible

        system_settings = SystemSettings.get_system_settings()
        form_init = {'ignore_cache': system_settings.ignore_cache}

        # Collect all fields organized by section, with section id 0
        # representing common report level fields
        fields_by_section = report.collect_fields_by_section()

        # Merge fields into a single dict for use by the Django Form # logic
        all_fields = SortedDict()
        [all_fields.update(c) for c in fields_by_section.values()]
        form = TableFieldForm(all_fields,
                              hidden_fields=report.hidden_fields,
                              initial=form_init)

        # Build a section map that indicates which section each field
        # belongs in when displayed
        section_map = []
        if fields_by_section[0]:
            section_map.append({'title': 'Common',
                                'parameters': fields_by_section[0]})

        for s in Section.objects.filter(report=report).order_by('position', 'title'):
            show = False
            for v in fields_by_section[s.id].values():
                if v.keyword not in (report.hidden_fields or []):
                    show = True
                    break

            if show:
                section_map.append({'title': s.title,
                                    'parameters': fields_by_section[s.id]})

        return render_to_response('report.html',
                                  {'report': report,
                                   'developer': system_settings.developer,
                                   'maps_version': system_settings.maps_version,
                                   'maps_api_key': system_settings.maps_api_key,
                                   'endtime': 'endtime' in form.fields,
                                   'form': form,
                                   'section_map': section_map,
                                   'show_sections': (len(section_map) > 1)},
                                  context_instance=RequestContext(request))

    def post(self, request, namespace=None, report_slug=None):
        # handle REST calls
        if namespace is None or report_slug is None:
            return self.http_method_not_allowed(request)

        logger.debug("Received POST for report %s, with params: %s" %
                     (report_slug, request.POST))

        try:
            report = Report.objects.get(namespace=namespace,
                                        slug=report_slug)
        except:
            raise Http404

        fields_by_section = report.collect_fields_by_section()
        all_fields = SortedDict()
        [all_fields.update(c) for c in fields_by_section.values()]
        form = TableFieldForm(all_fields, hidden_fields=report.hidden_fields,
                              data=request.POST, files=request.FILES)

        if form.is_valid():
            logger.debug('Form passed validation: %s' % form)
            formdata = form.cleaned_data
            logger.debug('Form cleaned data: %s' % formdata)

            # parse time and localize to user profile timezone
            timezone = pytz.timezone(request.user.timezone)
            form.apply_timezone(timezone)

            if formdata['debug']:
                logger.debug("Debugging report and rotating logs now ...")
                management.call_command('rotate_logs')

            logger.debug("Report %s validated form: %s" %
                         (report_slug, formdata))

            # setup definitions for each Widget
            definition = []

            # store datetime info about when report is being run
            # XXX move datetime format to preferences or somesuch
            now = datetime.datetime.now(timezone)
            definition.append({'datetime': str(date(now, 'jS F Y H:i:s')),
                               'timezone': str(timezone),
                               'debug': formdata['debug']})

            # create matrix of Widgets
            lastrow = -1
            rows = []
            for w in report.widgets().order_by('row', 'col'):
                if w.row != lastrow:
                    lastrow = w.row
                    rows.append([])
                rows[-1].append(Widget.objects.get_subclass(id=w.id))

            # populate definitions
            for row in rows:
                for w in row:
                    widget_def = {"widgettype": w.widgettype().split("."),
                                  "posturl": reverse('widget-job-list',
                                                     args=(report.namespace,
                                                           report.slug,
                                                           w.id)),
                                  "options": w.uioptions,
                                  "widgetid": w.id,
                                  "row": w.row,
                                  "width": w.width,
                                  "height": w.height,
                                  "criteria": w.criteria_from_form(form)
                                  }
                    definition.append(widget_def)

            logger.debug("Sending widget definitions for report %s: %s" %
                         (report_slug, definition))

            return HttpResponse(json.dumps(definition))
        else:
            # return form with errors attached in a HTTP 200 Error response
            return HttpResponse(str(form.errors), status=400)


class ReportCriteria(views.APIView):
    """ Handle requests for criteria fields.

        `get`  returns a json object of all criteria for the specified
               Report, or optionally for a specific widget within the Report

        `post` takes a criteria form and returns a json object of just
               the changed, or dynamic values
    """
    renderer_classes = (TemplateHTMLRenderer, JSONRenderer)

    def get(self, request, namespace=None, report_slug=None, widget_id=None):
        try:
            all_fields = SortedDict()

            if widget_id:
                w = Widget.objects.get(pk=widget_id)
                all_fields = w.collect_fields()
            else:
                report = Report.objects.get(namespace=namespace,
                                            slug=report_slug)
                fields_by_section = report.collect_fields_by_section()
                for c in fields_by_section.values():
                    all_fields.update(c)
        except:
            raise Http404

        form = TableFieldForm(all_fields, use_widgets=False)

        # create object from the tablefield keywords
        # then populate it with the initial data that got generated by default
        keys = form._tablefields.keys()
        criteria = dict(zip(keys, [None]*len(keys)))
        criteria.update(form.data)

        return HttpResponse(json.dumps(criteria))

    def post(self, request, namespace=None, report_slug=None):
        # handle REST calls
        if report_slug is None:
            return self.http_method_not_allowed(request)

        logger.debug("Received POST for report %s, with params: %s" %
                     (report_slug, request.POST))

        try:
            report = Report.objects.get(slug=report_slug)
        except:
            raise Http404

        fields_by_section = report.collect_fields_by_section()
        all_fields = SortedDict()
        [all_fields.update(c) for c in fields_by_section.values()]

        form = TableFieldForm(all_fields, hidden_fields=report.hidden_fields,
                              data=request.POST, files=request.FILES)

        response = []

        for field in form.dynamic_fields():
            response.append({'id': field.auto_id,
                             'html': str(field)})
        return HttpResponse(json.dumps(response))


class ReportWidgets(views.APIView):
    """ Return default criteria values for all widgets, with latest time,
        if applicable.

        Used in auto-run reports when specifying detailed criteria isn't
        necessary.
    """

    def get(self, request, namespace=None, report_slug=None):
        try:
            report = Report.objects.get(namespace=namespace,
                                        slug=report_slug)
        except:
            raise Http404

        # parse time and localize to user profile timezone
        timezone = pytz.timezone(request.user.timezone)
        now = datetime.datetime.now(timezone)

        # pin the endtime to a round interval if we are set to
        # reload periodically
        minutes = report.reload_minutes
        if minutes:
            trimmed = round_time(dt=now, round_to=60*minutes, trim=True)
            if now - trimmed > datetime.timedelta(minutes=15):
                now = trimmed
            else:
                now = round_time(dt=now, round_to=60*report.reload_minutes)

        widget_defs = []

        widget_defs.append({'datetime': str(date(now, 'jS F Y H:i:s')),
                            'timezone': str(timezone),
                            })
        for w in report.widgets().order_by('row', 'col'):
            # get default criteria values for widget
            # and set endtime to now, if applicable
            criteria = ReportCriteria.as_view()(request,
                                                w.section.report.namespace,
                                                w.section.report.slug,
                                                w.id)
            widget_criteria = json.loads(criteria.content)
            if 'endtime' in widget_criteria:
                widget_criteria['endtime'] = now.isoformat()

            # setup json definition object
            widget_def = {
                "widgettype": w.widgettype().split("."),
                "posturl": reverse('widget-job-list',
                                   args=(w.section.report.namespace,
                                         w.section.report.slug,
                                         w.id)),
                "options": w.uioptions,
                "widgetid": w.id,
                "row": w.row,
                "width": w.width,
                "height": w.height,
                "criteria": widget_criteria
            }

            widget_defs.append(widget_def)

        return HttpResponse(json.dumps(widget_defs))


class ReportTableList(generics.ListAPIView):
    model = Table
    serializer_class = TableSerializer

    def get(self, request, *args, **kwargs):
        report = Report.objects.get(namespace=kwargs['namespace'],
                                    slug=kwargs['report_slug'])
        qs = (table
              for section in report.section_set.all()
              for widget in section.widget_set.all()
              for table in widget.tables.all())
        serializer = TableSerializer(instance=qs)
        return Response(serializer.data)


class WidgetJobsList(views.APIView):

    parser_classes = (JSONParser,)

    def post(self, request, namespace, report_slug, widget_id, format=None):
        logger.debug("Received POST for report %s, widget %s: %s" %
                     (report_slug, widget_id, request.POST))

        try:
            report = Report.objects.get(namespace=namespace, slug=report_slug)
            widget = Widget.objects.get(id=widget_id)
        except:
            raise Http404

        req_json = json.loads(request.POST['criteria'])

        fields = widget.collect_fields()

        form = TableFieldForm(fields, use_widgets=False,
                              hidden_fields=report.hidden_fields,
                              include_hidden=True,
                              data=req_json, files=request.FILES)

        if not form.is_valid():
            raise ValueError("Widget internal criteria form is invalid:\n%s" %
                             (form.errors.as_text()))

        if form.is_valid():
            logger.debug('Form passed validation: %s' % form)
            formdata = form.cleaned_data
            logger.debug('Form cleaned data: %s' % formdata)

            # parse time and localize to user profile timezone
            timezone = pytz.timezone(request.user.timezone)
            form.apply_timezone(timezone)

            try:
                form_criteria = form.criteria()
                logger.debug('Form_criteria: %s' % form_criteria)
                job = Job.create(table=widget.table(),
                                 criteria=form_criteria)
                job.start()

                wjob = WidgetJob(widget=widget, job=job)
                wjob.save()

                logger.debug("Created WidgetJob %s for report %s (handle %s)" %
                             (str(wjob), report_slug, job.handle))

                return Response({"joburl": reverse('report-job-detail',
                                                   args=[namespace,
                                                         report_slug,
                                                         widget_id,
                                                         wjob.id])})
            except Exception as e:
                logger.exception("Failed to start job, an exception occurred")
                return HttpResponse(str(e), status=400)

        else:
            logger.error("form is invalid, entering debugger")
            from IPython import embed; embed()


class WidgetJobDetail(views.APIView):

    def get(self, request, namespace, report_slug, widget_id, job_id, format=None):
        wjob = WidgetJob.objects.get(id=job_id)

        job = wjob.job
        widget = wjob.widget

        if not job.done():
            # job not yet done, return an empty data structure
            logger.debug("%s: Not done yet, %d%% complete" % (str(wjob),
                                                              job.progress))
            resp = job.json()
        elif job.status == Job.ERROR:
            resp = job.json()
            logger.debug("%s: Job in Error state, deleting Job" % str(wjob))
            wjob.delete()
        else:
            try:
                i = importlib.import_module(widget.module)
                widget_func = i.__dict__[widget.uiwidget].process
                if widget.rows > 0:
                    tabledata = job.values()[:widget.rows]
                else:
                    tabledata = job.values()

                if tabledata is None or len(tabledata) == 0:
                    resp = job.json()
                    resp['status'] = Job.ERROR
                    resp['message'] = "No data returned"
                    logger.debug("%s marked Error: No data returned" %
                                 str(wjob))
                elif hasattr(i, 'authorized') and not i.authorized()[0]:
                    _, msg = i.authorized()
                    resp = job.json()
                    resp['data'] = None
                    resp['status'] = Job.ERROR
                    resp['message'] = msg
                    logger.debug("%s Error: module unauthorized for user %s"
                                 % (str(wjob), request.user))
                elif (hasattr(i, 'authorized') and
                      not Location.objects.all() and
                      not LocationIP.objects.all()):
                    # we are using a maps widget, but have no locations
                    resp = job.json()
                    resp['data'] = None
                    resp['status'] = Job.ERROR
                    msg = '''\
Geolocation data has not been loaded.
See <a href="https://support.riverbed.com/apis/steelscript/appfwk/configuration.html#locations">\
geolocation documentation</a> for more information.'''
                    resp['message'] = msg
                    logger.debug("%s Error: geo location data not loaded.")
                else:
                    data = widget_func(widget, job, tabledata)
                    resp = job.json(data)
                    logger.debug("%s complete" % str(wjob))

            except:
                logger.exception("Widget %s Job %s processing failed" %
                                 (widget.id, job.id))
                resp = job.json()
                resp['status'] = Job.ERROR
                ei = sys.exc_info()
                resp['message'] = str(traceback.format_exception_only(ei[0],
                                                                      ei[1]))

            wjob.delete()

        resp['message'] = cgi.escape(resp['message'])

        return HttpResponse(json.dumps(resp))



import pandas
import logging

logger = logging.getLogger(__name__)


def analysis_echo_criteria(query, tables, criteria, params):
    df = pandas.DataFrame([[str(k),str(v)]
                           for k,v in criteria.iteritems()],
                          columns=['key', 'value'])
    return df

def preprocess_field_choices(form, id, field_kwargs, params):
    field_kwargs['choices'] = [('val1', 'Value 1'),
                               ('val2', 'Value 2'),
                               ('val3', 'Value 3')]

def preprocess_field_choices_with_params(form, id, field_kwargs, params):
    choices = []
    for i in range(params['start'], params['end']+1):
        val = params['prefix'] + '_val' + str(i)
        choices.append((val, val))
        
    field_kwargs['choices'] = choices

def preprocess_changesecond(form, id, field_kwargs, params):
    first_val = form.get_field_value('first', id)
    field_kwargs['choices'] = [(first_val + ('-%d' % i), 'Choice %d' % i) for i in range(3)]
        
def postprocess_field_compute(form, id, criteria, params):
    s = 0
    for f in params['fields']:
        s = s + int(criteria[f])

    criteria[id] = s

def sharedfields_compute(form, id, criteria, params):
    criteria[id] = str(int(criteria['x']) * 2 + int(params['factor']))
    
def postprocesserrors_compute(form, id, criteria, params):
    if criteria['error'] == 'syntax':
        adsf
    else:
        criteria['x'] = 1
        



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from rest_framework import serializers

from steelscript.appfwk.apps.devices.models import Device


class DeviceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Device



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import os
from random import choice

from steelscript.commands.steel import (BaseCommand, prompt, console, debug,
                                        shell, check_git, ShellFailed)
from steelscript.appfwk.project.utils import link_pkg_dir, link_pkg_files


LOCAL_CONTENT = """
from steelscript.appfwk.project.settings import *

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
DATAHOME = os.getenv('DATAHOME', PROJECT_ROOT)
DATA_CACHE = os.path.join(DATAHOME, 'data', 'datacache')
INITIAL_DATA = os.path.join(DATAHOME, 'data', 'initial_data')
REPORTS_DIR = os.path.join(PROJECT_ROOT, 'reports')

STATIC_ROOT = os.path.join(PROJECT_ROOT, 'static')
MEDIA_ROOT = DATA_CACHE

# Optionally add additional applications specific to this project instance

LOCAL_APPS = (
    # additional apps can be listed here
)
INSTALLED_APPS += LOCAL_APPS

# Configure database for development or production.

DATABASES = {
    'default': {
        # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'ENGINE': 'django.db.backends.sqlite3',

        # Path to database file if using sqlite3.
        # Database name for others
        'NAME': os.path.join(DATAHOME, 'data', 'project.db'),

        'USER': '',     # Not used with sqlite3.
        'PASSWORD': '', # Not used with sqlite3.
        'HOST': '',     # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',     # Set to empty string for default. Not used with sqlite3.
    }
}

# Setup loggers to local directory
LOGGING['handlers']['logfile']['filename'] = os.path.join(DATAHOME, 'logs', 'log.txt')
LOGGING['handlers']['backend-log']['filename'] = os.path.join(DATAHOME, 'logs', 'log-db.txt')

# To enable syslog handling instead of local logging, uncomment next block of LOGGING
# statements

# remove these loggers since the configuration will attempt to write the
# files even if they don't have a logger declared for them
#LOGGING['disable_existing_loggers'] = True
#LOGGING['handlers'].pop('logfile')
#LOGGING['handlers'].pop('backend-log')
#
#LOGGING['loggers'] = {
#    'django.db.backends': {
#        'handlers': ['null'],
#        'level': 'DEBUG',
#        'propagate': False,
#    },
#    '': {
#        'handlers': ['syslog'],
#        'level': 'INFO',
#        'propagate': True,
#    },
#}

"""

LOCAL_FOOTER = """
# Add other settings customizations below, which will be local to this
# machine only, and not recorded by git. This could include database or
# other authentications, LDAP settings, or any other overrides.

# For example LDAP configurations, see the file
# `project/ldap_example.py`.
"""

GITIGNORE = """
*~
*.pyc
*.swp
.DS_Store

data/
example-configs/
logs/
media
thirdparty
static/
"""


class Command(BaseCommand):
    help = 'Install new local App Framework project'

    def add_options(self, parser):
        parser.add_option('-d', '--dir', action='store',
                          help='Optional path for new project location')
        parser.add_option('-v', '--verbose', action='store_true',
                          help='Extra verbose output')
        parser.add_option('--nogit', action='store_true',
                          help='Do not initialize project as new git repo')

    def debug(self, msg, newline=False):
        if self.options.verbose:
            debug(msg, newline=newline)

    def mkdir(self, dirname):
        """Creates directory if it doesn't already exist."""
        if not os.path.exists(dirname):
            os.mkdir(dirname)

    def create_local_settings(self, dirname):
        """Creates local settings configuration."""

        secret = ''.join([
            choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)')
            for i in range(50)
        ])

        fname = os.path.join(dirname, 'local_settings.py')
        if not os.path.exists(fname):
            console('Writing local settings %s ... ' % fname, newline=False)
            with open(fname, 'w') as f:
                f.write(LOCAL_CONTENT)
                f.write("SECRET_KEY = '%s'\n" % secret)
                f.write(LOCAL_FOOTER)
            console('done.')
        else:
            console('Skipping local settings generation.')

    def create_project_directory(self, dirpath):
        """Creates project directory and copies/links necessary files."""
        console('Creating project directory %s ...' % dirpath)
        self.mkdir(dirpath)

        # link manage.py and media directories
        for p in ('manage.py', 'media', 'thirdparty'):
            link_pkg_dir('steelscript.appfwk.apps',
                         '../' + p,
                         os.path.join(dirpath, p),
                         buf=self.debug)

        # copy and make folders
        self.mkdir(os.path.join(dirpath, 'logs'))

        datapath = os.path.join(dirpath, 'data')
        self.mkdir(datapath)
        link_pkg_dir('steelscript.appfwk.apps',
                     '../initial_data',
                     os.path.join(datapath, 'initial_data'),
                     symlink=False,
                     buf=self.debug)
        self.mkdir(os.path.join(datapath, 'datacache'))

        # copy example-configs
        link_pkg_files('steelscript.appfwk.apps',
                       '../project/example-configs/*',
                       os.path.join(dirpath, 'example-configs'),
                       symlink=False,
                       buf=self.debug)

        # copy sample locations
        link_pkg_files('steelscript.appfwk.apps',
                       'geolocation/sample_location*',
                       os.path.join(dirpath, 'example-configs'),
                       symlink=False,
                       buf=self.debug)

    def initialize_git(self, dirpath):
        """If git installed, initialize project folder as new repo.
        """
        try:
            check_git()
        except ShellFailed:
            return

        # we have git, lets make a repo
        shell('git init', msg='Initializing project as git repo',
              cwd=dirpath)
        fname = os.path.join(dirpath, '.gitignore')
        with open(fname, 'w') as f:
            f.write(GITIGNORE)
        shell('git add .',
              msg=None,
              cwd=dirpath)
        shell('git commit -a -m "Initial commit."',
              msg='Creating initial git commit',
              cwd=dirpath)

    def collectreports(self, dirpath):
        shell('python manage.py collectreports -v 3 --trace',
              msg='Collecting default reports',
              cwd=dirpath)

    def main(self):
        console('Generating new Application Framework project ...')

        dirpath = self.options.dir
        while not dirpath:
            default = os.path.join(os.getcwd(), 'appfwk_project')
            dirpath = prompt('\nEnter path for project files',
                             default=default)

        dirpath = os.path.abspath(dirpath)
        if os.path.exists(dirpath):
            console('Project directory already exists, aborting.')
            return

        self.create_project_directory(dirpath)
        self.create_local_settings(dirpath)
        self.collectreports(dirpath)
        if not self.options.nogit:
            self.initialize_git(dirpath)

        console('\n*****\n')
        console('App Framework project created.')
        console("Change to that directory and run "
                "'steel appfwk init' to initialize the project.")



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from django import forms

from steelscript.appfwk.apps.report.models import Report, Widget

import logging
logger = logging.getLogger(__name__)

DURATIONS = ('Default', '15 min', '1 hour', 
             '2 hours', '4 hours', '12 hours', '1 day',
             '1 week', '4 weeks')


class ReportDetailForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ReportDetailForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Report


class WidgetDetailForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(WidgetDetailForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Widget
        exclude = ['tables', 'module', 'uiwidget', 'uioptions']



import logging

import pandas
from steelscript.common.timeutils import datetime_to_seconds


logger = logging.getLogger(__name__)

def analysis_generate_data(query, tables, criteria, params):
    t0 = datetime_to_seconds(criteria.starttime)
    t1 = datetime_to_seconds(criteria.endtime)

    data = []
    for t in range(t0, t1, params['source_resolution']):
        data.append([t, 1])

    df = pandas.DataFrame(data, columns=['time', 'value'])
    df['time'] = pandas.DatetimeIndex(df['time']*1000000000)
    return df




# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from django.conf.urls import patterns, url
from rest_framework.urlpatterns import format_suffix_patterns

from steelscript.appfwk.apps.datasource.views import TableList, TableDetail
from steelscript.appfwk.apps.datasource.views import TableColumnList, TableJobList
from steelscript.appfwk.apps.datasource.views import ColumnList, ColumnDetail
from steelscript.appfwk.apps.datasource.views import JobList, JobDetail, JobDetailData


urlpatterns = patterns(
    '',

    url(r'^tables/$',
        TableList.as_view(),
        name='table-list'),

    url(r'^tables/(?P<pk>[0-9]+)/$',
        TableDetail.as_view(),
        name='table-detail'),

    url(r'^tables/(?P<pk>[0-9]+)/columns/$',
        TableColumnList.as_view(),
        name='table-column-list'),

    url(r'^tables/(?P<pk>[0-9]+)/jobs/$',
        TableJobList.as_view(),
        name='table-job-list'),

    url(r'^columns/$',
        ColumnList.as_view(),
        name='column-list'),

    url(r'^columns/(?P<pk>[0-9]+)/$',
        ColumnDetail.as_view(),
        name='column-detail'),

    url(r'^jobs/$',
        JobList.as_view(),
        name='job-list'),

    url(r'^jobs/(?P<pk>[0-9]+)/$',
        JobDetail.as_view(),
        name='job-detail'),

    url(r'^jobs/(?P<pk>[0-9]+)/data/$',
        JobDetailData.as_view(),
        name='job-detail-data'),
)

urlpatterns = format_suffix_patterns(urlpatterns)



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import math


class NiceScale:

    def __init__(self, minval, maxval, maxticks=10, forcezero=False, zerothresh=0.2):
        self.minval = minval if minval else 0
        self.maxval = maxval if maxval else 0
        self.maxticks = maxticks
        self.zerothresh = zerothresh
        self.forcezero = forcezero
        self.calculate()
        
    #
    # Calculate and update values for tick spacing and nice
    # minimum and maximum data points on the axis.
    #
    def calculate(self):
        if (self.forcezero or (self.minval > 0) and 
                (self.maxval > 0) and 
                ((float(self.maxval - self.minval) / self.maxval) > self.zerothresh)):
            self.minval = 0

        vrange = self.maxval - self.minval
        if vrange == 0:
            self.tickSpacing = 1
            self.niceMin = 0
            self.niceMax = 1
            self.numTicks = 1
        else:
            valrange = self.niceNum(vrange, False)
            self.tickSpacing = self.niceNum(float(valrange) / (self.maxticks - 1), True)
            self.niceMin = math.floor(self.minval / self.tickSpacing) * self.tickSpacing
            self.niceMax = math.ceil(self.maxval / self.tickSpacing) * self.tickSpacing
            self.numTicks = 1 + round((self.niceMax - self.niceMin) / self.tickSpacing)
        
    #
    # Returns a "nice" number approximately equal to range Rounds
    # the number if round = True Takes the ceiling if round = False.
    #
    # @param valrange the data range
    # @param round whether to round the result
    # @return a "nice" number to be used for the data range
    
    def niceNum(self, valrange, round):
        if valrange == 0:
            return 0

        exponent = math.floor(math.log10(valrange))
        fraction = valrange / math.pow(10, exponent)
        
        if (round):
            if (fraction < 1.5):
                niceFraction = 1
            elif (fraction < 3):
                niceFraction = 2
            elif (fraction < 7):
                niceFraction = 5
            else:
                niceFraction = 10
        else:
            if (fraction <= 1):
                niceFraction = 1
            elif (fraction <= 2):
                niceFraction = 2
            elif (fraction <= 5):
                niceFraction = 5
            else:
                niceFraction = 10
            
        return niceFraction * math.pow(10, exponent)
    
    def dump(self):
        a = []
        x = self.niceMin
        while x <= self.niceMax:
            a.append("%.4f" % x)
            x += self.tickSpacing
        print "(%.4f - %.4f @ %d) => %d [%s]" % (self.minval,
                                                 self.maxval,
                                                 self.maxticks, 
                                                 self.numTicks, 
                                                 ', '.join(a))

    
if __name__ == "__main__":
    def test(minval, maxval, maxticks=10):
        n = NiceScale(minval, maxval, maxticks)
        n.dump()
        
    #test(100,500)
    #test(100,500,5)
    #test(100,500,4)
    #test(92,156)
    #test(0.09,.9)
    #test(0.2, 0.28)
    #test(1, 19)
    test(0.101, 0.119)



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from django import forms

from steelscript.netprofiler.core import _constants
from steelscript.appfwk.apps.devices.models import Device


def get_device_choices(device_type):
    devices = Device.objects.filter(module=device_type)
    return [(d.id, d.name) for d in devices]


def get_groupbys():
    return [('', '---')] + sorted(((v, k) for k,v in _constants.groupbys.iteritems()))


def get_realms():
    return [('', '---')] + [(c, c.title().replace('_', ' ')) for c in _constants.realms]


def get_centricities():
    return ('', '---'), ('hos', 'host'), ('int', 'interface')


class NetProfilerInputForm(forms.Form):
    device = forms.ChoiceField(choices=get_device_choices('netprofiler'))
    realm = forms.ChoiceField(choices=get_realms())
    centricity = forms.ChoiceField(choices=get_centricities())
    groupby = forms.ChoiceField(choices=get_groupbys())


class NetSharkInputForm(forms.Form):
    device = forms.ChoiceField(choices=get_device_choices('netshark'))



from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw

from . import criteria_functions as funcs

report = Report(title='Criteria Two Reports - 2')
report.save()

TableField.create(keyword='k2', label='Key 2', obj=report, initial='r2')

# Section
section = Section.create(report=report, title='Section')
section.save()

# Table
a = AnalysisTable.create('test-criteria-tworeports-2', tables={},
                         function=funcs.analysis_echo_criteria)
TableField.create(keyword='k1', label='Key 1', obj=a, initial='r1')

a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table 2')



from django import forms

from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.appfwk.libs.fields import Function
from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.report.modules import raw
from . import criteria_functions as funcs

report = Report(title='Criteria Changing',
                field_order =['first', 'second'])
report.save()

section = Section(report=report, title='Section 0')
section.save()

TableField.create('first', 'First Choice', section,
                  field_cls=forms.ChoiceField,
                  field_kwargs={'choices': (('a', 'Option A'),
                                            ('b', 'Option B'))})

TableField.create('second', 'Second Choice', section,
                  field_cls=forms.ChoiceField,
                  pre_process_func=Function(funcs.preprocess_changesecond),
                  dynamic=True)

a = AnalysisTable.create('test-criteria-changingchoices', tables={},
                         function=funcs.analysis_echo_criteria)
a.add_column('key', 'Key', iskey=True, datatype="string")
a.add_column('value', 'Value', datatype="string")

raw.TableWidget.create(section, a, 'Table')



# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.plugins.builtin.whois.datasource.whois import WhoisTable

from steelscript.appfwk.apps.datasource.modules.analysis import AnalysisTable
from steelscript.netprofiler.appfwk.datasources.netprofiler import NetProfilerGroupbyTable
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

# helper libraries

#
# NetProfiler report
#

report = Report.create("Whois", position=11)

report.add_section()

# Define a Table that gets external hosts by avg bytes
table = NetProfilerGroupbyTable.create('5-hosts', groupby='host', duration='1 hour',
                                    filterexpr='not srv host 10/8 and not srv host 192.168/16')

table.add_column('host_ip', 'IP Addr', iskey=True, datatype='string')
table.add_column('avg_bytes', 'Avg Bytes', units='B/s', sortdesc=True)


#report.add_widget(yui3.TableWidget, table, "Table", width=12)

# Create an Analysis table that calls the 'whois' function to craete a link to
# 'whois'
whoistable = WhoisTable.create('5-whois-hosts',
                               tables={'t': table})

report.add_widget(yui3.TableWidget, whoistable, "Link table", width=12)
